diff mbox series

[v2] btrfs: zoned: introduce a minimal zone size and reject mount

Message ID 8aa15bbbacbafa2ab77c01bfdfdabe65d6bfa606.1652457157.git.johannes.thumshirn@wdc.com (mailing list archive)
State New, archived
Headers show
Series [v2] btrfs: zoned: introduce a minimal zone size and reject mount | expand

Commit Message

Johannes Thumshirn May 13, 2022, 3:52 p.m. UTC
Zoned devices are expected to have zone sizes in the range of 1-2GB for
ZNS SSDs and SMR HDDs have zone sizes of 256MB, so there is no need to
allow arbitrarily small zone sizes on btrfs.

But for testing purposes with emulated devices it is sometimes desirable
to create devices with as small as 4MB zone size to uncover errors.

So use 4MB as the smallest possible zone size and reject mounts of devices
with a smaller zone size.

Signed-off-by: Johannes Thumshirn <johannes.thumshirn@wdc.com>
---
 fs/btrfs/zoned.c | 15 ++++++++++++---
 1 file changed, 12 insertions(+), 3 deletions(-)

Comments

David Sterba May 13, 2022, 5:37 p.m. UTC | #1
On Fri, May 13, 2022 at 08:52:52AM -0700, Johannes Thumshirn wrote:
> Zoned devices are expected to have zone sizes in the range of 1-2GB for
> ZNS SSDs and SMR HDDs have zone sizes of 256MB, so there is no need to
> allow arbitrarily small zone sizes on btrfs.
> 
> But for testing purposes with emulated devices it is sometimes desirable
> to create devices with as small as 4MB zone size to uncover errors.
> 
> So use 4MB as the smallest possible zone size and reject mounts of devices
> with a smaller zone size.
> 
> Signed-off-by: Johannes Thumshirn <johannes.thumshirn@wdc.com>

Thanks, added to misc-next.

> ---
>  fs/btrfs/zoned.c | 15 ++++++++++++---
>  1 file changed, 12 insertions(+), 3 deletions(-)
> 
> diff --git a/fs/btrfs/zoned.c b/fs/btrfs/zoned.c
> index 1b1b310c3c51..d9579d4ec0f2 100644
> --- a/fs/btrfs/zoned.c
> +++ b/fs/btrfs/zoned.c
> @@ -51,11 +51,13 @@
>  #define BTRFS_MIN_ACTIVE_ZONES		(BTRFS_SUPER_MIRROR_MAX + 5)
>  
>  /*
> - * Maximum supported zone size. Currently, SMR disks have a zone size of
> - * 256MiB, and we are expecting ZNS drives to be in the 1-4GiB range. We do not
> - * expect the zone size to become larger than 8GiB in the near future.
> + * Minimum / maximum supported zone size. Currently, SMR disks have a zone
> + * size of 256MiB, and we are expecting ZNS drives to be in the 1-4GiB range.
> + * We do not expect the zone size to become larger than 8GiB or smaller than
> + * 4MiB in the near future.
>   */
>  #define BTRFS_MAX_ZONE_SIZE		SZ_8G
> +#define BTRFS_MIN_ZONE_SIZE		(4 * SZ_1M)

I've checked if the SZ_4M constant exists, it does so I'll use it.
Nikolay Borisov May 13, 2022, 6:42 p.m. UTC | #2
On 13.05.22 г. 18:52 ч., Johannes Thumshirn wrote:
> Zoned devices are expected to have zone sizes in the range of 1-2GB for
> ZNS SSDs and SMR HDDs have zone sizes of 256MB, so there is no need to
> allow arbitrarily small zone sizes on btrfs.
> 
> But for testing purposes with emulated devices it is sometimes desirable
> to create devices with as small as 4MB zone size to uncover errors.
> 
> So use 4MB as the smallest possible zone size and reject mounts of devices
> with a smaller zone size.
> 
> Signed-off-by: Johannes Thumshirn <johannes.thumshirn@wdc.com>
> ---
>   fs/btrfs/zoned.c | 15 ++++++++++++---
>   1 file changed, 12 insertions(+), 3 deletions(-)
> 
> diff --git a/fs/btrfs/zoned.c b/fs/btrfs/zoned.c
> index 1b1b310c3c51..d9579d4ec0f2 100644
> --- a/fs/btrfs/zoned.c
> +++ b/fs/btrfs/zoned.c
> @@ -51,11 +51,13 @@
>   #define BTRFS_MIN_ACTIVE_ZONES		(BTRFS_SUPER_MIRROR_MAX + 5)
>   
>   /*
> - * Maximum supported zone size. Currently, SMR disks have a zone size of
> - * 256MiB, and we are expecting ZNS drives to be in the 1-4GiB range. We do not
> - * expect the zone size to become larger than 8GiB in the near future.
> + * Minimum / maximum supported zone size. Currently, SMR disks have a zone
> + * size of 256MiB, and we are expecting ZNS drives to be in the 1-4GiB range.
> + * We do not expect the zone size to become larger than 8GiB or smaller than
> + * 4MiB in the near future.
>    */
>   #define BTRFS_MAX_ZONE_SIZE		SZ_8G
> +#define BTRFS_MIN_ZONE_SIZE		(4 * SZ_1M)

nit: we already have SZ_4M

>   
>   #define SUPER_INFO_SECTORS	((u64)BTRFS_SUPER_INFO_SIZE >> SECTOR_SHIFT)
>   
> @@ -402,6 +404,13 @@ int btrfs_get_dev_zone_info(struct btrfs_device *device, bool populate_cache)
>   				 zone_info->zone_size, BTRFS_MAX_ZONE_SIZE);
>   		ret = -EINVAL;
>   		goto out;
> +	} else if (zone_info->zone_size < BTRFS_MIN_ZONE_SIZE) {
> +		btrfs_err_in_rcu(fs_info,
> +		"zoned: %s: zone size %llu smaller than supported minimum %u",
> +				 rcu_str_deref(device->name),
> +				 zone_info->zone_size, BTRFS_MIN_ZONE_SIZE);
> +		ret = -EINVAL;
> +		goto out;
>   	}
>   
>   	nr_sectors = bdev_nr_sectors(bdev);
Nikolay Borisov May 13, 2022, 6:46 p.m. UTC | #3
On 13.05.22 г. 21:42 ч., Nikolay Borisov wrote:
> 
> 
> On 13.05.22 г. 18:52 ч., Johannes Thumshirn wrote:
>> Zoned devices are expected to have zone sizes in the range of 1-2GB for
>> ZNS SSDs and SMR HDDs have zone sizes of 256MB, so there is no need to
>> allow arbitrarily small zone sizes on btrfs.
>>
>> But for testing purposes with emulated devices it is sometimes desirable
>> to create devices with as small as 4MB zone size to uncover errors.
>>
>> So use 4MB as the smallest possible zone size and reject mounts of 
>> devices
>> with a smaller zone size.
>>
>> Signed-off-by: Johannes Thumshirn <johannes.thumshirn@wdc.com>
>> ---
>>   fs/btrfs/zoned.c | 15 ++++++++++++---
>>   1 file changed, 12 insertions(+), 3 deletions(-)
>>
>> diff --git a/fs/btrfs/zoned.c b/fs/btrfs/zoned.c
>> index 1b1b310c3c51..d9579d4ec0f2 100644
>> --- a/fs/btrfs/zoned.c
>> +++ b/fs/btrfs/zoned.c
>> @@ -51,11 +51,13 @@
>>   #define BTRFS_MIN_ACTIVE_ZONES        (BTRFS_SUPER_MIRROR_MAX + 5)
>>   /*
>> - * Maximum supported zone size. Currently, SMR disks have a zone size of
>> - * 256MiB, and we are expecting ZNS drives to be in the 1-4GiB range. 
>> We do not
>> - * expect the zone size to become larger than 8GiB in the near future.
>> + * Minimum / maximum supported zone size. Currently, SMR disks have a 
>> zone
>> + * size of 256MiB, and we are expecting ZNS drives to be in the 
>> 1-4GiB range.
>> + * We do not expect the zone size to become larger than 8GiB or 
>> smaller than
>> + * 4MiB in the near future.
>>    */
>>   #define BTRFS_MAX_ZONE_SIZE        SZ_8G
>> +#define BTRFS_MIN_ZONE_SIZE        (4 * SZ_1M)
> 
> nit: we already have SZ_4M

... and David has already seen it so this is noop :)
> 
>>   #define SUPER_INFO_SECTORS    ((u64)BTRFS_SUPER_INFO_SIZE >> 
>> SECTOR_SHIFT)
>> @@ -402,6 +404,13 @@ int btrfs_get_dev_zone_info(struct btrfs_device 
>> *device, bool populate_cache)
>>                    zone_info->zone_size, BTRFS_MAX_ZONE_SIZE);
>>           ret = -EINVAL;
>>           goto out;
>> +    } else if (zone_info->zone_size < BTRFS_MIN_ZONE_SIZE) {
>> +        btrfs_err_in_rcu(fs_info,
>> +        "zoned: %s: zone size %llu smaller than supported minimum %u",
>> +                 rcu_str_deref(device->name),
>> +                 zone_info->zone_size, BTRFS_MIN_ZONE_SIZE);
>> +        ret = -EINVAL;
>> +        goto out;
>>       }
>>       nr_sectors = bdev_nr_sectors(bdev);
>
diff mbox series

Patch

diff --git a/fs/btrfs/zoned.c b/fs/btrfs/zoned.c
index 1b1b310c3c51..d9579d4ec0f2 100644
--- a/fs/btrfs/zoned.c
+++ b/fs/btrfs/zoned.c
@@ -51,11 +51,13 @@ 
 #define BTRFS_MIN_ACTIVE_ZONES		(BTRFS_SUPER_MIRROR_MAX + 5)
 
 /*
  * Maximum supported zone size. Currently, SMR disks have a zone size of
  * 256MiB, and we are expecting ZNS drives to be in the 1-4GiB range. We do not
  * expect the zone size to become larger than 8GiB in the near future.
  */
 #define BTRFS_MAX_ZONE_SIZE		SZ_8G
+#define BTRFS_MIN_ZONE_SIZE		(4 * SZ_1M)
 
 #define SUPER_INFO_SECTORS	((u64)BTRFS_SUPER_INFO_SIZE >> SECTOR_SHIFT)
 
@@ -402,6 +404,13 @@  int btrfs_get_dev_zone_info(struct btrfs_device *device, bool populate_cache)
 				 zone_info->zone_size, BTRFS_MAX_ZONE_SIZE);
 		ret = -EINVAL;
 		goto out;
+	} else if (zone_info->zone_size < BTRFS_MIN_ZONE_SIZE) {
+		btrfs_err_in_rcu(fs_info,
+		"zoned: %s: zone size %llu smaller than supported minimum %u",
+				 rcu_str_deref(device->name),
+				 zone_info->zone_size, BTRFS_MIN_ZONE_SIZE);
+		ret = -EINVAL;
+		goto out;
 	}
 
 	nr_sectors = bdev_nr_sectors(bdev);
@@ -51,11 +51,13 @@ 
 
 /*
- * Maximum supported zone size. Currently, SMR disks have a zone size of
- * 256MiB, and we are expecting ZNS drives to be in the 1-4GiB range. We do not
- * expect the zone size to become larger than 8GiB in the near future.
+ * Minimum / maximum supported zone size. Currently, SMR disks have a zone
+ * size of 256MiB, and we are expecting ZNS drives to be in the 1-4GiB range.
+ * We do not expect the zone size to become larger than 8GiB or smaller than
+ * 4MiB in the near future.
  */
 #define BTRFS_MAX_ZONE_SIZE		SZ_8G